include <eased_shapes.scad>
include <screws_and_rods.scad>

$fn = 256;

bevels = 1.5;
small_bevels = .5;

clearance = .1;

//roller_od = 2 * 25.4;
roller_od = 3 * 25.4;
roller_id = 26;
roller_h = 2 * 25.4;

bearing = [12,32,10];

curve_dx = 1.5;

assembly(true);

module assembly ( curved = false )
{
  difference () {
    if ( curved ) {
      curved_roller();
    } else {
      roller();
    }

    bearing_cutout();
  }
}

module roller ( extra = false )
{
  r = roller_od / 2;
  h = roller_h + 2 * bevels;
  b = bevels + ( extra ? curve_dx : 0 );

  eased_cylinder ( r = r, h = h,
    top = b, top_type = es_bevel,
    bottom = b, bottom_type = es_bevel,
    center = true );
}

module curved_roller ()
{

  intersection () {
    curve();
    roller(true);
  }
}

function square ( x )
  = x * x;

/* Formula for the curve is based
   on a right triangle where the
   radius (r) intersects the cylinder at
   the top or bottom, the long leg (x)
   reaches to the edge of the cylinder
   at the centerline, and the short leg
   (y) is half the height of the cylinder
   (not including bevels). Thus, where
   dx = the bulge of the curve beyond
   the cylinder,

   r = x + dx
   r^2 = x^2 + y^2
   (x + dx)^2 = x^2 + y^2
   x^2 + 2xdx + dx^2 = x^2 + y^2
   2xdx + dx^2 = y^2
   2xdx = y^2 - dx^2
   x = (y^2 - dx^2) / 2dx
*/
   
module curve ()
{
  $fn = 1024;

  roller_r = roller_od / 2;

  y = roller_h / 2;
  x = ( square(y) - square(curve_dx) ) /
      ( 2 * curve_dx );
  r = x + curve_dx;

  dx = r - roller_r;
  
  c = r * 2 + 2;

  rotate_extrude ( angle = 360 ) {
    difference () {
      translate([dx,0,0])
      circle(r=r);

      translate([0,-c/2,0])
      square(c);
    }
  }
}

module bearing_cutout ()
{
  r = roller_id / 2;
  h = roller_h + 2 * bevels;
  cr = bearing.y / 2 + clearance;
  ch = bearing.z + bevels;

  dz = - h / 2;

  mirror_copy([0,0,1])
  translate([0,0,dz]) {
    sr_screw ( r = r, h = h,
      counter_r = cr, counter_h = ch,
      counter_transition = 60,
      bevels = bevels );
  }
}